#ifndef BatchLab_AIDA_Session_h
#define BatchLab_AIDA_Session_h

#include <AIDA/IAnalysisFactory.h>

#include <Lib/Printer.h>
#include <Lib/Out.h>
#include <Lib/Session.h>
#include <Lib/LibraryManager.h>
#include <Lib/System.h>
#include <Lib/smanip.h>
#include <Lib/Arguments.h>
#include <Lib/Debug.h>

#include <BatchLab/Core/AnalysisFactory.h>

namespace BatchLab {

// The AIDA_Session handles a concrete Slash::Core::ISession and 
// is also a proxy to a BatchLab::AnalysisFactory.

class AIDA_Session : public virtual AIDA::IAnalysisFactory {
public: //AIDA/IAnalysisFactory
  virtual AIDA::ITreeFactory* createTreeFactory();
  virtual AIDA::IHistogramFactory* createHistogramFactory(AIDA::ITree&);
  virtual AIDA::ITupleFactory* createTupleFactory(AIDA::ITree&);
  virtual AIDA::IFunctionFactory* createFunctionFactory(AIDA::ITree&);
  virtual AIDA::IPlotterFactory* createPlotterFactory
    (int,char**,const std::string&,const std::string&);
  virtual AIDA::IDataPointSetFactory* createDataPointSetFactory(AIDA::ITree&);
  virtual AIDA::IFitFactory* createFitFactory();
public:
  AIDA_Session();
  virtual ~AIDA_Session();
  bool isValid() const;
private:
  Slash::Core::IWriter* fPrinter;
  Slash::Core::ISession* fSession;
  AIDA::IAnalysisFactory* fAnalysisFactory;
  int fVerboseLevel;
  bool fBalance;
};

}


#endif
