#ifndef BatchLab_Filters_h
#define BatchLab_Filters_h

// Inheritance :
#include <AIDA/IFilter.h>

#include <BatchLab/Core/amanip.h>

namespace BatchLab {

class AcceptAll : public virtual AIDA::IFilter {
public: //AIDA/IFilter
  virtual bool initialize(AIDA::ITuple&){ return true;}
  virtual bool accept() const {return true;}
  virtual std::string expression() const { return "";}
public:
  AcceptAll(){}
  virtual ~AcceptAll(){}
};

class Filter2D : public virtual AIDA::IFilter {
public: //AIDA/IFilter
  virtual bool initialize(AIDA::ITuple&){ return true;}
  virtual bool accept() const {
    double x,y;
    if(!BatchLab::amanip::tuple_getDouble(fPrinter,fTuple,fX,x)) return false;
    if(!BatchLab::amanip::tuple_getDouble(fPrinter,fTuple,fY,y)) return false;
    if( (fXmin<=x)&&(x<fXmax) &&
        (fYmin<=y)&&(y<fYmax) ){
      return true;
    }
    return false;
  }
  virtual std::string expression() const { return "";}
public:
  Filter2D(Slash::Core::IWriter& aPrinter,
           const AIDA::ITuple& aTuple,
           int aX,double aXmin,double aXmax,
           int aY,double aYmin,double aYmax)
  :fPrinter(aPrinter)
  ,fTuple(aTuple)
  ,fX(aX),fXmin(aXmin),fXmax(aXmax)
  ,fY(aY),fYmin(aYmin),fYmax(aYmax)
  {}
  virtual ~Filter2D(){}
private:
  Slash::Core::IWriter& fPrinter;
  const AIDA::ITuple& fTuple;
  int fX;
  double fXmin,fXmax;
  int fY;
  double fYmin,fYmax;
};

class Filter3D : public virtual AIDA::IFilter {
public: //AIDA/IFilter
  virtual bool initialize(AIDA::ITuple&){ return true;}
  virtual bool accept() const {
    double x,y,z;
    if(!BatchLab::amanip::tuple_getDouble(fPrinter,fTuple,fX,x)) return false;
    if(!BatchLab::amanip::tuple_getDouble(fPrinter,fTuple,fY,y)) return false;
    if(!BatchLab::amanip::tuple_getDouble(fPrinter,fTuple,fZ,z)) return false;
    if( (fXmin<=x)&&(x<fXmax) &&
        (fYmin<=y)&&(y<fYmax) &&
        (fZmin<=z)&&(z<fZmax) ){
      return true;
    }
    return false;
  }
  virtual std::string expression() const { return "";}
public:
  Filter3D(Slash::Core::IWriter& aPrinter,
           const AIDA::ITuple& aTuple,
           int aX,double aXmin,double aXmax,
           int aY,double aYmin,double aYmax,
           int aZ,double aZmin,double aZmax)
  :fPrinter(aPrinter)
  ,fTuple(aTuple)
  ,fX(aX),fXmin(aXmin),fXmax(aXmax)
  ,fY(aY),fYmin(aYmin),fYmax(aYmax)
  ,fZ(aZ),fZmin(aZmin),fZmax(aZmax)
  {}
  virtual ~Filter3D(){}
private:
  Slash::Core::IWriter& fPrinter;
  const AIDA::ITuple& fTuple;
  int fX;
  double fXmin,fXmax;
  int fY;
  double fYmin,fYmax;
  int fZ;
  double fZmin,fZmax;
};

}

#endif
