#ifndef OnX_Session_h
#define OnX_Session_h

// Inheritance :
#include <Slash/Core/ISession.h>
#include <Lib/StreamBuffer.h>

#include <vector>

namespace Slash {namespace Core {class IWriter;}}

#include <Lib/Parameters.h>

namespace OnX {

class StreamDestination : public Lib::StreamDestination {
public:
  StreamDestination(Slash::Core::IWriter&);
  virtual ~StreamDestination();
  int receive(const std::string&);
private:
  Slash::Core::IWriter& fPrinter;
};


class Session : public virtual Slash::Core::ISession {
public: //Slash::Core::ISession
  virtual std::string name() const;
  virtual void addManager(Slash::Core::IManager*);
  virtual void removeManager(Slash::Core::IManager*);
  virtual Slash::Core::IManager* findManager(const std::string&) const;
  virtual void destroyManager(const std::string&);
  virtual Slash::Core::IWriter& printer();
  virtual bool load(const std::string&,const std::string&);
  virtual void redirectOutput(bool);
  virtual bool setParameter(const std::string&,const std::string&);
  virtual bool parameterValue(const std::string&,std::string&);
  virtual std::vector<std::string> availableParameters() const;
  virtual void flush();
  virtual void setVerboseLevel(int);
  virtual int verboseLevel() const;
  virtual unsigned int numberOfManagers() const;
  virtual const Slash::Core::IManager* manager(unsigned int) const;
  virtual std::vector<std::string> managerNames() const;
  virtual std::vector< std::pair<std::string,std::string> > arguments() const;
public:
  Session(const std::string&,int);
  virtual ~Session();
  void coutToOnX();
  void coutToDefault();
  void stdoutToOnX();
  void stdoutToDefault();
  //virtual void executeScript(const std::string&,const std::string&);
  void setArguments(const std::vector< std::pair<std::string,std::string> >&);
private:
  std::vector<Slash::Core::IManager*> fManagers;
  Slash::Core::IWriter* fPrinter;
  // cout redirection :
  streambuf* fOutOld;
  Lib::StreamBuffer* fOutNew;
  StreamDestination* fOutDestination;
  // cerr redirection :
  streambuf* fErrOld;
  Lib::StreamBuffer* fErrNew;
  StreamDestination* fErrDestination;
  // stdout redirection :
  std::string fOutDefault;
  std::string fOutName;
  void* fOutFile; //Do not expose FILE*
  // stderr redirection :
  std::string fErrDefault;
  std::string fErrName;
  void* fErrFile; //Do not expose FILE*
  Lib::Parameters<std::string,std::string> fParameters;
  int fVerboseLevel;
  std::vector< std::pair<std::string,std::string> > fArguments;
  std::string fName;
};

}

#endif


