#ifndef OnX_Helpers_Inventor_h
#define OnX_Helpers_Inventor_h

//
//  This file contains static helper methods.
// It is intended to be included in OnX Callbacks file
// to ease callbacks programming.
//

// This file contains no Inventor related helpers.
// See OnX/Helpers/Inventor.h for Inventor related ones.

// Slash :
#include <Slash/UI/IUI.h>
#include <Slash/UI/IWidget.h>
#include <Slash/UI/ISoViewer.h>

// HEPVis :
#include <HEPVis/nodekits/SoPage.h>

// Lib :
#include <Lib/Cast.h>

//////////////////////////////////////////////////////////////////////////////
/// Slash::UI::ISoViewer /////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
inline Slash::UI::ISoViewer* ui_SoViewer(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::IWidget* widget = aUI.currentWidget();
  if(!widget) return 0;
  return Lib_Cast(*widget,Slash::UI::ISoViewer);
}
//////////////////////////////////////////////////////////////////////////////
inline Slash::UI::ISoViewer* ui_SoViewer(
 Slash::UI::IWidget& aWidget
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  return Lib_Cast(aWidget,Slash::UI::ISoViewer);
}
//////////////////////////////////////////////////////////////////////////////
inline SoPage* ui_SoPage(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::ISoViewer* soViewer = ui_SoViewer(aUI);
  if(!soViewer) return 0;
  SoNode* soNode = soViewer->sceneGraph();
  if(!soNode) return 0;
  if(!soNode->isOfType(SoPage::getClassTypeId())) return 0;
  return (SoPage*)soNode;    
}
//////////////////////////////////////////////////////////////////////////////
inline SoRegion* ui_SoRegion(
 Slash::UI::IUI& aUI
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  SoPage* soPage = ui_SoPage(aUI);
  if(!soPage) return 0;
  return soPage->currentRegion();
}

//////////////////////////////////////////////////////////////////////////////
inline SoPage* ui_SoPage(
 Slash::UI::IWidget& aWidget
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::ISoViewer* soViewer = Lib_Cast(aWidget,Slash::UI::ISoViewer);
  if(!soViewer) return 0;
  SoNode* soNode = soViewer->sceneGraph();
  if(!soNode) return 0;
  if(!soNode->isOfType(SoPage::getClassTypeId())) return 0;
  return (SoPage*)soNode;    
}

#endif
