#ifndef OnXLab_Environment_h
#define OnXLab_Environment_h

#include <Slash/Core/IWriter.h>

#include <Lib/System.h>
#include <Lib/argZero.h>
#include <Lib/Out.h>

//WARNING : code not put in a Environment.cxx put in OnXLabCore
//          to avoid dependencies of OnXLabInventor, etc.. over 
//          OnXLabCore and then OnXCore.

//////////////////////////////////////////////////////////////////////////////
inline bool check_ONXLABROOT(
 const std::string& aArg0
,Slash::Core::IWriter& aPrinter
,bool aWarn = true
)
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string fs = Lib::System::fileSeparator();
  std::string to = "Resources"+fs+"OnXLab";

  std::string dir_root;
  bool is_Mac_app;
  bool status = 
    Lib::argZero::checkEnv(aArg0,"ONXLABROOT",to,aPrinter,aWarn,
                        dir_root,is_Mac_app);
  if(status) {
    if(dir_root=="") {
      // Assume that ONXLABROOT is defined through the dev setup.
      return true;
    } else {
      // Activated through an app in a OSC binary kit.
      // Have to define ONXLABROOT.
    }
  } else {
    // May be activated from a user app that uses a OSC binary kit.
    // To detect this situation, we look for the AIDA_HOME_DIR 
    // that should be defined by the [aida-]setup of these kits.
    std::string AIDA_HOME_DIR;
    if(!Lib::System::getenv("AIDA_HOME_DIR",AIDA_HOME_DIR)) {
      Lib::Out out(aPrinter);
      out << "check_ONXLABROOT :"
          << " cant guess where OpenScientist had been installed."
          << Lib::endl;
      return false;
    } else {
      dir_root = AIDA_HOME_DIR;
    }
  }

  if(!Lib::System::putenv
        ("ONXLABROOT",dir_root+fs+"Resources"+fs+"OnXLab")) return false;

  if(!Lib::System::envPathAppend("PYTHONPATH",
       dir_root+fs+"Resources"+fs+"OnXLab"+fs+"scripts"+fs+"Python")) 
    return false;
  if(!Lib::System::envPathAppend("PYTHONPATH",dir_root+fs+"bin")) return false;

  return true;
}

#endif
